@@ -1,6 +1,6 @@
 # 2016. 1. 10 by Hans Roh hansroh@gmail.com
 
-__version__ = "0.7.8.11"
+__version__ = "0.7.10.2"
 version_info = tuple (map (lambda x: not x.isdigit () and x or int (x),  __version__.split (".")))
 
 import os, sys
@@ -14,7 +14,7 @@ from .dbapi import dbpool
 from .client import adns, asynconnect
 from .lib.athreads.fifo import await_fifo
 from . import client, dbapi
-from aquests.protocols.dns.asyndns import async_dns
+from aquests.protocols.dns import asyndns
 from .protocols.http import localstorage as ls
 from .protocols.http import request_handler, response as http_response
 from .protocols import http2
@@ -138,7 +138,8 @@ def configure (
 	_initialized = True
 
 def _reque_first (request):
-	global _que	
+	global _que
+	
 	_que.first (request)	
 
 def handle_status_401 (response):
@@ -149,9 +150,8 @@ def handle_status_401 (response):
 	_logger ("authorization failed, %s" % response.url, "info")		
 	request = response.request	
 	request.reauth_count = 1	
-	del response
 	_reque_first (request)	
-
+	
 def handle_status_3xx (response):
 	global _allow_redirects	, _que
 	
@@ -176,24 +176,29 @@ def handle_status_3xx (response):
 	
 	#_logger ("%s redirected to %s from %s" % (response.status_code, newloc, oldloc), "info")	
 	# DO NOT use relocated response.request, it is None
-	del response
 	_reque_first (request)
 	
 def _request_finished (handler):
 	global _cb_gateway, _currents, _concurrent, _finished_total, _logger, _bytesrecv,_force_h1
 	
+	req_id = handler.request.meta ['req_id']	
+	try: 
+		_currents.pop (req_id)
+	except KeyError:
+		pass
+			
 	if isinstance (handler, dbo_request.Request):
 		response = handler	
-		_currents.pop (response.meta ['req_id'])
+		
 	else:
-		response = response_builder.HTTPResponse (handler)
-		_currents.pop (response.meta ['req_id'])
+		response = response_builder.HTTPResponse (handler.response)		
 		
 		try:
 			for handle_func in (handle_status_401, handle_status_3xx):
 				response = handle_func (response)
 				if not response:
-					return
+					# re-requested
+					return req_if_queue (req_id)					
 		except:
 			_logger.trace ()
 		
@@ -206,15 +211,16 @@ def _request_finished (handler):
 	except:		
 		_logger.trace ()	
 	
-	# clearing memory
-	handler.request.meta = None
-	del response
+	req_if_queue (req_id)
+	
+def req_if_queue (req_id):
+	global _logger, _currents
 	
 	try:
-		qsize () and _req ()		
+		qsize () and _req ()
 	except RecursionError:
 		try: 
-			_currents.pop (handler.request.meta ['req_id'])
+			_currents.pop (req_id)
 		except KeyError: 
 			pass	
 		_logger ("too many error occured, failed requeueing", "fail")
@@ -247,7 +253,8 @@ def _req ():
 			asyncon = dbpool.get (server, dbname, auth, "*" + _method)
 			req = request_builder.make_dbo (_method, server, dbmethod, params, dbname, auth, meta, _logger)
 		else:
-			asyncon = dbpool.get (req.server, req.dbname, req.auth, "*" + req.dbtype)				
+			asyncon = dbpool.get (req.server, req.dbname, req.auth, "*" + req.dbtype)
+			
 		_currents [meta ['req_id']] = [0, req.server]
 		req.set_callback (_request_finished)
 		asyncon.execute (req)
@@ -260,16 +267,17 @@ def _req ():
 				req = request_builder.make_ws (_method, url, params, auth, headers, meta, proxy, _logger)
 			else:
 				req = request_builder.make_http (_method, url, params, auth, headers, meta, proxy, _logger)			
+		
 		else:
 			asyncon = socketpool.get (req.uri)		
-		_currents [meta ['req_id']] = [0, req.uri]
-		
+
+		_currents [meta ['req_id']] = [0, req.uri]		
 		handler = req.handler (asyncon, req, _request_finished)
 		if asyncon.get_proto () and asyncon.isconnected ():
 			asyncon.handler.handle_request (handler)
 		else:
-			handler.handle_request ()	
-
+			handler.handle_request ()		
+	
 def workings ():
 	global _currents
 	return 	len (_currents)
@@ -284,7 +292,7 @@ def qsize ():
 
 def mapsize ():
 	return len (asyncore.socket_map)
-	
+
 def countfin ():	
 	global _finished_total
 	return _finished_total
@@ -296,7 +304,7 @@ def countcli ():
 def concurrent ():
 	global _concurrent
 	return _concurrent
-
+		
 def fetchall ():
 	global _workers, _logger, _que, _timeout, _max_conns, _bytesrecv, _concurrent, _finished_total, _max_conns, _force_h1	
 	
@@ -316,12 +324,13 @@ def fetchall ():
 	target_socks = min (_workers, qsize ())
 	for i in range (target_socks):
 		_req ()
-			
+	lifetime.poll_dns (True)
+		
 	if not _force_h1 and http2.MAX_HTTP2_CONCURRENT_STREAMS > 1:
 		# wait all availabale	
 		while qsize ():
 			lifetime.lifetime_loop (os.name == "nt" and 1.0 or _timeout / 2.0, 1)
-			if sum ([1 for conn in asyncore.socket_map.values () if not isinstance (conn, async_dns) and conn.get_proto () in H2_PROTOCOLS and conn.connected and not conn.isactive ()]) == _workers:
+			if sum ([1 for conn in asyncore.socket_map.values () if conn.get_proto () in H2_PROTOCOLS and conn.connected and not conn.isactive ()]) == _workers:
 				#_logger ('%d connection(s) created' % target_socks, 'info')
 				break
 			
@@ -332,7 +341,8 @@ def fetchall ():
 		_max_conns = max (_max_conns, mapsize ())	
 		#print ('--', len (_currents), mapsize (), qsize ())
 		if not mapsize ():
-			break
+			break	
+		lifetime.poll_dns (True)
 		lifetime.lifetime_loop (os.name == "nt" and 1.0 or _timeout / 2.0, 1)
 	
 	#for each in _currents:
@@ -372,18 +382,18 @@ def _add (method, url, params = None, auth = None, headers = {}, callback = None
 	meta ['req_id'] = _que.req_id
 	meta ['req_method'] = method
 	meta ['req_callback'] = callback
-	host = urlparse (url) [1].split (":")[0]
+	_que.add ((method, url, params, auth, headers, meta, proxy))
 	
 	# DNS query for caching and massive
 	if not lifetime._polling:
+		host = urlparse (url) [1].split (":")[0]
 		if _dns_reqs < _workers and host not in _dns_query_req:
 			_dns_query_req [host] = None
 			_dns_reqs += 1
 			adns.query (host, "A", callback = lambda x: None)		
-		if mapsize ():
-			lifetime.poll_fun_wrap (0.05)
-		
-	_que.add ((method, url, params, auth, headers, meta, proxy))
+		lifetime.poll_dns (True)
+	
+	#print ('~~~~~~~~~~~~~~~', asyndns.pool.connections)
 	
 #----------------------------------------------------
 # Add Reuqest (protocols.*.request) Object
@@ -407,7 +417,6 @@ def options (*args, **karg):
 def upload (*args, **karg):
 	_add ('upload', *args, **karg)
 
-
 def get (*args, **karg):
 	_add ('get', *args, **karg)
 
