@@ -16,6 +16,7 @@ from ..lib.athreads.fifo import await_fifo
 from ..lib.ssl_ import resolve_cert_reqs, resolve_ssl_version, create_urllib3_context
 from collections import deque
 from ..protocols.http import respcodes
+import random
 
 DEBUG = 0
 	
@@ -44,7 +45,6 @@ class AsynConnect (asynchat.async_chat):
 		
 		self.auth = None
 		self.proxy = False
-		
 		self.initialize_connection ()
 		self._closed = False
 		self.backend = False
@@ -79,9 +79,10 @@ class AsynConnect (asynchat.async_chat):
 		return self.auth
 				
 	def close (self):
-		if self._closed:
-			return
-		
+		# sometimes socket is not closed at once
+		# possibly related with SSL socket
+		# then prevent doble callbacking in request_handler
+				
 		if self.socket:
 			# self.socket is still None, when DNS not found
 			asynchat.async_chat.close (self)
@@ -92,15 +93,16 @@ class AsynConnect (asynchat.async_chat):
 		self.incoming = []
 		self.producer_fifo.clear()
 		self._proto = None
-		self._closed = True
+		self._closed = True		
 			
 		if not self.handler:
 			# return to the pool
-			return self.set_active (False)		
+			return self.set_active (False)
+		
 		if not self.errcode:
 			# disconnect intentionally
 			return
-			
+		
 		handler, self.handller = self.handler, None
 		keep_active = False
 		try:
@@ -109,21 +111,21 @@ class AsynConnect (asynchat.async_chat):
 			self.trace ()
 		
 		if not keep_active:
+			#print (handler.request.meta ['sid'], 'not keepalive...')
 			self.set_active (False)	
-			if self.errcode not in (704, 712):
+			if self.errcode not in (704, 712, 722):
 				# controlled shutdown
 				self.logger (
 					".....socket %s has been closed (reason: %d)" % ("%s:%d" % self.address, self.errcode),
 					"info"
 				)
 		# DO NOT Change any props, because may be request has been restarted	
-	
+		
 	def end_tran (self):
 		if DEBUG:
 			self.logger ('end_tran {rid:%s} %s' % (self.handler.request.meta.get ('req_id', -1), self.handler.request.uri), 'debug')
-		if not self.backend:
+		if not self.backend:			
 			self.del_channel ()
-		self.handler = None
 		self.set_active (False)
 		if not self.keep_connect:
 			self.disconnect ()
@@ -237,7 +239,7 @@ class AsynConnect (asynchat.async_chat):
 		if fd in map:
 			del map[fd]
             			
-	def create_socket (self, family, type):		
+	def create_socket (self, family, type):
 		self.family_and_type = family, type
 		sock = socket.socket (family, type)
 		sock.setblocking (0)
@@ -247,9 +249,9 @@ class AsynConnect (asynchat.async_chat):
 		if adns.query:
 			adns.query (self.address [0], "A", callback = self.continue_connect)
 		else:			
-			self.continue_connect (True)
+			self.continue_connect ()
 		
-	def continue_connect (self, answer = None):		
+	def continue_connect (self, answer = None):
 		self.initialize_connection ()
 		
 		if not adns.query:
@@ -259,14 +261,17 @@ class AsynConnect (asynchat.async_chat):
 			return
 		
 		ipaddr = answer and answer [-1]["data"] or None		
-		port = self.address [1]
-		
+		#print (self.handler.request.meta ['sid'], ipaddr, 'continue_connect...')
 		if not ipaddr:			
 			return self.handle_close (704)			
-												
+		else:	
+			port = self.address [1]
+		
 		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)
-		try: asynchat.async_chat.connect (self, (ipaddr, port))									
-		except:	self.handle_error (714)
+		try: 
+			asynchat.async_chat.connect (self, (ipaddr, port))
+		except:	
+			self.handle_error (714)
 		
 	def recv (self, buffer_size):
 		try:
@@ -314,9 +319,13 @@ class AsynConnect (asynchat.async_chat):
 			self.disconnect ()
 		
 	def set_timeout (self, timeout = 10):
-		self.zombie_timeout = timeout
+		# CAUTION: used at proxy.tunnel_handler
+		self.zombie_timeout = timeout		
+	
+	def set_keep_alive (self, keep_alive = 10):
+		self.keep_alive = keep_alive
 		
-	def handle_connect (self):
+	def handle_connect (self):		
 		if hasattr (self.handler, "has_been_connected"):		
 			self.handler.has_been_connected ()
 					
@@ -339,12 +348,13 @@ class AsynConnect (asynchat.async_chat):
 	
 	def maintern (self, object_timeout):		
 		if time.time () - self.event_time > object_timeout:
-			if self.handler and hasattr (self.handler, "enter_shutdown_process"):
-				self.handler.enter_shutdown ()
-				return True
+			if self.handler:
+				if hasattr (self.handler, "control_shutdown"):
+					self.handler.control_shutdown ()
+				self.handle_close (722)
 			else:	
 				self.disconnect ()
-				return True		
+			return True
 		return False
 		
 	# proxy POST need no init_send
@@ -367,7 +377,7 @@ class AsynConnect (asynchat.async_chat):
 	
 	def handle_abort (self):
 		self.handler = None
-		self.close ()
+		self.close ()		
 		
 	def handle_close (self, code = 700, msg = ""):
 		if code == 0: msg = ""
@@ -380,15 +390,17 @@ class AsynConnect (asynchat.async_chat):
 							
 	def collect_incoming_data (self, data):
 		if not self.handler:
-			self.logger ("recv data but no handler, droping data %d" % len (data), "warn")
-			self.disconnect ()
+			if self.connected:
+				self.logger ("recv data but no handler, droping data %d" % len (data), "warn")
+				self.disconnect ()
 			return
 		self.handler.collect_incoming_data (data)
 	
 	def found_terminator (self):
 		if not self.handler:
-			self.logger ("found terminator but no handler", "warn")
-			self.disconnect ()
+			if self.connected:
+				self.logger ("found terminator but no handler", "warn")
+				self.disconnect ()
 			return # already closed
 		self.handler.found_terminator ()
 	
@@ -420,9 +432,11 @@ class AsynConnect (asynchat.async_chat):
 		# IMP: call add_channel () AFTER push()	otherwise threading issue will be raised
 		try:
 			if not self.connected:				
+				#print (self.handler.request.meta ['sid'], 'connecting...')
 				self.connect ()
 								
 			elif not self.backend:
+				#print (self.handler.request.meta ['sid'], 'add channel...')
 				self.add_channel ()
 				
 		except:
@@ -439,7 +453,7 @@ class AsynSSLConnect (AsynConnect):
 		if not self._handshaking:
 			err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
 			if err != 0:
-				raise socket.error(err, os.strerror(err))
+				raise OSError(err, asyncore._strerror(err))
 				
 			ssl_context = create_urllib3_context(ssl_version=resolve_ssl_version(None), cert_reqs=resolve_cert_reqs(None))
 			if self.ac_negotiate_http2:
@@ -468,12 +482,8 @@ class AsynSSLConnect (AsynConnect):
 			if not self._handshaked and not self.handshake ():
 				return
 		except:
-			self.handle_error (713)
-			return
-					
-		# handshaking done
-		self.handle_connect()
-		self.connected = True
+			return self.handle_error (713)
+		AsynConnect.handle_connect_event (self)		
 		
 	def recv (self, buffer_size):
 		if self._closed:
