@@ -6,6 +6,8 @@ import bisect
 import socket
 import time
 import types
+from .protocols.dns import asyndns
+
 try:
 	from pympler import muppy, summary, tracker
 except ImportError:
@@ -54,10 +56,13 @@ class Maintern:
 def maintern_gc (now):
 	gc.collect ()
 	
-def maintern_zombie_channel (now):
+def maintern_zombie_channel (now, map = None):
 	global _killed_zombies
-		
-	for channel in list(asyncore.socket_map.values()):
+	
+	if map is None:
+		map = asyncore.socket_map
+			
+	for channel in list(map.values()):
 		if hasattr (channel, "handle_timeout"):
 			try:
 				# +3 is make gap between server & client
@@ -179,13 +184,11 @@ def remove_notsocks (map):
 
 def poll_fun_wrap (timeout, map = None):
 	global _logger
-	
+
 	if map is None:
-		map = asyncore.socket_map
-	
+		map = asyncore.socket_map	
 	try:		
-		poll_fun (timeout, map)		
-		
+		poll_fun (timeout, map)
 	except (TypeError, OSError) as why:
 		# WSAENOTSOCK
 		remove_notsocks (map)
@@ -209,7 +212,19 @@ def poll_fun_wrap (timeout, map = None):
 	except:
 		_logger and _logger.trace ()
 		raise
-	
+
+def poll_dns (exhaust = False):
+	map = asyndns.socket_map
+	if exhaust:
+		while asyndns.pool:
+			asyncore.loop (0.5, map = map, count = 1)						
+			asyndns.pool.maintern (time.time ())
+			
+	else:	
+		if asyndns.pool:
+			# asyndns.pool.maintern () is scheduled by skitai
+			asyncore.loop (0.1, map = map, count = 2)
+		
 def lifetime_loop (timeout = 30.0, count = 0):
 	global _last_maintern
 	global _maintern_interval
